require_relative 'spec_helper'

describe SessionManager::Handler do
 
  let (:login) { {login: 'login'}.to_json}
  let (:api_key) { {api_key: '01'}.to_json}
  let (:user_option) { {user: true, app: false}}
  let (:app_option) { {user: false, app: true}}
  let (:user_session) {{login: 'login',token: 'jdijenfdeifiirifffjr'}.to_json}
  let (:app_session) {{api_key: '01',token: 'jdijenfdeifiirifffjr'}.to_json} 

  before :each do
    redis=Redis.current
    redis.del('users_sessions')
    redis.del('applications_sessions')
    redis.sadd('users_sessions',user_session)
    redis.sadd('applications_sessions',app_session)

  end

  describe ".create_session" do
    context "when a user's session is created" do
      it { expect((SessionManager::Handler.create_session(login,user_option))[:code]).to eq 200}       
    end 

    context "When an app's session is created" do
        it { expect((SessionManager::Handler.create_session(api_key,app_option))[:code]).to eq 200 }
    end

    context "When option is invalid" do
        option = {u: true,a:false}
        it { expect((SessionManager::Handler.create_session(api_key,option))[:code]).to eq 400 }

        option = {user: true,app: true}
        it { expect((SessionManager::Handler.create_session(api_key,option))[:code]).to eq 400 }

        option = " "
        it { expect((SessionManager::Handler.create_session(api_key,option))[:code]).to eq 400 }

        option = {user: nil,app:false}
        it { expect((SessionManager::Handler.create_session(api_key,option))[:code]).to eq 400 }

        option = {user: "tata",app: "toto"}
        it { expect((SessionManager::Handler.create_session(api_key,option))[:code]).to eq 400 }
    end

    context "When the object's column is invalid" do    
        api_key = {api_key: nil}.to_json
        option = {user: false,app: true}
        it { expect((SessionManager::Handler.create_session(api_key,option))[:code]).to eq 400 }

        api_key ={api_key: 1}.to_json 
        it { expect((SessionManager::Handler.create_session(api_key,option))[:code]).to eq 400 }

        login = {login: 'fatou'}
        opt ={user: true,app:false}
        it { expect((SessionManager::Handler.create_session(login,opt))[:code]).to eq 400 }
               api_key = {ap: 'invalid'}.to_json
        it { expect((SessionManager::Handler.create_session(api_key,option))[:code]).to eq 400 }

        it { expect((SessionManager::Handler.create_session(nil,option))[:code]).to eq 400 }

        it { expect((SessionManager::Handler.create_session(" ",option))[:code]).to eq 400 }
    end

    context "when the given attribute doesn't correspond to the option" do
        api_key = {api_key: 'hhfrijijfri'}.to_json
        option = {user: true,app: false}
        it { expect((SessionManager::Handler.create_session(api_key,option))[:code]).to eq 400 }

    end
  end

  describe ".verify_session" do
      context "when a user's session is found" do
          it { expect((SessionManager::Handler.verify_session(user_session,user_option))[:code]).to eq 200}
      end
      

      context "when a user's session is not found" do
         not_user_session = {login: 'login',token: ''}.to_json 
        it { expect((SessionManager::Handler.verify_session(not_user_session,user_option))[:code]).to eq 400}
      end

      context "when the user's session is not json" do
         user_session = {login: 'login',token: 'jdijenfdeifiirr'} 
        it { expect((SessionManager::Handler.verify_session(user_session,user_option))[:code]).to eq 400}
      end

      context "when the param's keys are invalid" do
         user_session = {logn: 'login',toke: 'jdijenfdeifiirr'} 
        it { expect((SessionManager::Handler.verify_session(user_session,user_option))[:code]).to eq 400}
      end

      context "when the param's values are not valid" do
         user_session = {login: 1,token: 'jdijenfdeifiirr'} 
        it { expect((SessionManager::Handler.verify_session(user_session,user_option))[:code]).to eq 400}

         user_session = {login: '',token: nil} 
        it { expect((SessionManager::Handler.verify_session(user_session,user_option))[:code]).to eq 400}
      end

      
      context "when app's session is found" do
          it { expect((SessionManager::Handler.verify_session(app_session,app_option))[:code]).to eq 200}
      end

      context "when app's session is not found" do
        not_app_session = {api_key: '02',token: 'jdijenfdeifi'}.to_json
        it { expect((SessionManager::Handler.verify_session(not_app_session,app_option))[:code]).to eq 400}
      end
  end

  describe ".delete_session" do
      context "when a user's session is deleted" do
        it { expect((SessionManager::Handler.delete_session(user_session,user_option))[:code]).to eq 200}
      end

      context "when a user's session is not deleted" do
         not_user_session = {login: 'login',token: 'jdijenfd'}.to_json 
        it { expect((SessionManager::Handler.delete_session(not_user_session,user_option))[:code]).to eq 400}
      end

      context "when a user's session is not valid" do
         not_user_session = {login: 'login',token: 'jdijenfdeifiir'} 
        it { expect((SessionManager::Handler.delete_session(not_user_session,user_option))[:code]).to eq 400}

         not_user_session = {logn: 'login',tken: 'jdijenfdeifiir'}.to_json 
        it { expect((SessionManager::Handler.delete_session(not_user_session,user_option))[:code]).to eq 400}

         not_user_session = {login: nil,token: nil}.to_json 
        it { expect((SessionManager::Handler.delete_session(not_user_session,user_option))[:code]).to eq 400}

         not_user_session = {login: 1,token: 'jdijenfdeifiir'}.to_json 
        it { expect((SessionManager::Handler.delete_session(not_user_session,user_option))[:code]).to eq 400}

        not_user_session = {login: '',token: 'jdijenfdeifiir'}.to_json 
        it { expect((SessionManager::Handler.delete_session(not_user_session,user_option))[:code]).to eq 400}
 
        it { expect((SessionManager::Handler.delete_session(nil,user_option))[:code]).to eq 400}
        
        it { expect((SessionManager::Handler.delete_session(nil,nil))[:code]).to eq 400}
        
      end

      context "when a app's session is found" do
          it { expect((SessionManager::Handler.delete_session(app_session,app_option))[:code]).to eq 200}
      end

      context "when a app's session is not found" do
        not_app_session = {api_key: '02',token: 'jdijenfdeifiirr'}.to_json
          it { expect((SessionManager::Handler.delete_session(not_app_session,app_option))[:code]).to eq 400}
      end
  end
   

end
